/**
 *
 */
package org.gcube.portlets.user.speciesdiscovery.shared;

import java.io.Serializable;
import java.util.Date;

import com.google.gwt.user.client.rpc.IsSerializable;




/**
 * The Class JobGisLayerModel.
 *
 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
 * Feb 9, 2017
 */
public class JobGisLayerModel implements Serializable, IsSerializable{

	/**
	 *
	 */
	private static final long serialVersionUID = -8982510983374359781L;

	public static final String JOBNAME = "Layer Title";
	public static final String JOBINDENTIFIER = "jobIdentifier";
	public static final String STARTTIME = "Start Time";
	public static final String SUBMITTIME = "Submit Time";
	public static final String PROGRESS = "Progress";
	public static final String ENDTIME = "End Time";
	public static final String ELAPSEDTIME = "Elapsed Time";

	public static final String PERCENTAGE = "Percentage";
	public static final String DESCRIPTION = "Description";
	public static final String STATUS = "Status";

	private String jobIdentifier;
	private String jobName;
	private DownloadState downloadState;
	protected Date startTime;
	protected Date submitTime;
	protected Date endTime;
	protected String elapsedTime;
	private String layerDescription;
	private float percentage;
	private long totalPoints;
	private long completedPoints;
	private String layerUUID; //This is the result

	/**
	 * Instantiates a new job gis layer model.
	 */
	public JobGisLayerModel() {
	}


	/**
	 * Instantiates a new job gis layer model.
	 *
	 * @param jobIdentifier the job identifier
	 * @param layerTitle the layer title
	 * @param downloadState the download state
	 * @param completedPoints the completed points
	 * @param totalPoints the total points
	 */
	public JobGisLayerModel(String jobIdentifier, String layerTitle, DownloadState downloadState, long completedPoints, long totalPoints) {
		this.jobIdentifier = jobIdentifier;
		this.jobName = layerTitle;
		this.downloadState = downloadState;
		this.completedPoints = completedPoints;
		this.totalPoints = totalPoints;
	}




	/**
	 * @param jobIdentifier
	 * @param jobName
	 * @param downloadState
	 * @param startTime
	 * @param submitTime
	 * @param endTime
	 * @param elapsedTime
	 * @param layerDescription
	 * @param percentage
	 * @param totalPoints
	 * @param completedPoints
	 */
	public JobGisLayerModel(
		String jobIdentifier, String jobName, DownloadState downloadState,
		Date startTime, Date submitTime, Date endTime, String elapsedTime,
		String layerDescription, long completedPoints,
		long totalPoints) {

		this.jobIdentifier = jobIdentifier;
		this.jobName = jobName;
		this.downloadState = downloadState;
		this.startTime = startTime;
		this.submitTime = submitTime;
		this.endTime = endTime;
		this.elapsedTime = elapsedTime;
		this.layerDescription = layerDescription;
		this.totalPoints = totalPoints;
		this.completedPoints = completedPoints;
		this.percentage = completedPoints!=0?completedPoints*100/totalPoints:0;
	}


	/**
	 * @return the jobIdentifier
	 */
	public String getJobIdentifier() {

		return jobIdentifier;
	}



	/**
	 * @return the jobName
	 */
	public String getJobName() {

		return jobName;
	}



	/**
	 * @return the downloadState
	 */
	public DownloadState getDownloadState() {

		return downloadState;
	}



	/**
	 * @return the startTime
	 */
	public Date getStartTime() {

		return startTime;
	}



	/**
	 * @return the submitTime
	 */
	public Date getSubmitTime() {

		return submitTime;
	}



	/**
	 * @return the endTime
	 */
	public Date getEndTime() {

		return endTime;
	}



	/**
	 * @return the elapsedTime
	 */
	public String getElapsedTime() {

		return elapsedTime;
	}



	/**
	 * @return the layerDescription
	 */
	public String getLayerDescription() {

		return layerDescription;
	}



	/**
	 * @return the percentage
	 */
	public float getPercentage() {

		return percentage;
	}



	/**
	 * @return the totalPoints
	 */
	public long getTotalPoints() {

		return totalPoints;
	}



	/**
	 * @return the completedPoints
	 */
	public long getCompletedPoints() {

		return completedPoints;
	}



	/**
	 * @param jobIdentifier the jobIdentifier to set
	 */
	public void setJobIdentifier(String jobIdentifier) {

		this.jobIdentifier = jobIdentifier;
	}



	/**
	 * @param jobName the jobName to set
	 */
	public void setJobName(String jobName) {

		this.jobName = jobName;
	}



	/**
	 * @param downloadState the downloadState to set
	 */
	public void setDownloadState(DownloadState downloadState) {

		this.downloadState = downloadState;
	}



	/**
	 * @param startTime the startTime to set
	 */
	public void setStartTime(Date startTime) {

		this.startTime = startTime;
	}



	/**
	 * @param submitTime the submitTime to set
	 */
	public void setSubmitTime(Date submitTime) {

		this.submitTime = submitTime;
	}



	/**
	 * @param endTime the endTime to set
	 */
	public void setEndTime(Date endTime) {

		this.endTime = endTime;
	}



	/**
	 * @param elapsedTime the elapsedTime to set
	 */
	public void setElapsedTime(String elapsedTime) {

		this.elapsedTime = elapsedTime;
	}



	/**
	 * @param layerDescription the layerDescription to set
	 */
	public void setLayerDescription(String layerDescription) {

		this.layerDescription = layerDescription;
	}



	/**
	 * @param percentage the percentage to set
	 */
	public void setPercentage(float percentage) {

		this.percentage = percentage;
	}



	/**
	 * @param totalPoints the totalPoints to set
	 */
	public void setTotalPoints(int totalPoints) {

		this.totalPoints = totalPoints;
	}



	/**
	 * @param completedPoints the completedPoints to set
	 */
	public void setCompletedPoints(int completedPoints) {

		this.completedPoints = completedPoints;
	}


	/**
	 * @return the layerUUID
	 */
	public String getLayerUUID() {

		return layerUUID;
	}



	/**
	 * @param layerUUID the layerUUID to set
	 */
	public void setLayerUUID(String layerUUID) {

		this.layerUUID = layerUUID;
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {

		StringBuilder builder = new StringBuilder();
		builder.append("JobGisLayerModel [jobIdentifier=");
		builder.append(jobIdentifier);
		builder.append(", jobName=");
		builder.append(jobName);
		builder.append(", downloadState=");
		builder.append(downloadState);
		builder.append(", startTime=");
		builder.append(startTime);
		builder.append(", submitTime=");
		builder.append(submitTime);
		builder.append(", endTime=");
		builder.append(endTime);
		builder.append(", elapsedTime=");
		builder.append(elapsedTime);
		builder.append(", layerDescription=");
		builder.append(layerDescription);
		builder.append(", percentage=");
		builder.append(percentage);
		builder.append(", totalPoints=");
		builder.append(totalPoints);
		builder.append(", completedPoints=");
		builder.append(completedPoints);
		builder.append(", layerUUID=");
		builder.append(layerUUID);
		builder.append("]");
		return builder.toString();
	}

}
